package PureMessage::Service::postgresql;
use strict;
use base 'PureMessage::Service';
use PureMessage::Database::PostgreSQL::Service qw(db_start db_stop 
                                                  db_restart db_status);
use ActiveState::Handy qw(run);

sub Available {
    my $class = shift;
    my @avail;
    push(@avail, $class->new(name => "PostgreSQL"));
    return @avail;
}

sub _bg {
    my $pid = fork();
    die "Can't fork: $!" unless defined $pid;
    return $pid if $pid;

    open(STDIN, "</dev/null");
    open(STDOUT, ">/dev/null");
    open(STDERR, ">/dev/null");
    return 0;
}

sub start {
    my($self, %opts) = @_;
    if ($opts{bg}) {
        if (my $pid = _bg()) {
           return $pid;
        }
    }
    db_start();
    exit if $opts{bg};
}

sub stop {
    my($self, %opts) = @_;
    if ($opts{bg}) {
        if (my $pid = _bg()) {
            return $pid;
        }
    }
    db_stop();
    exit if $opts{bg};
}

sub restart {
    my($self, %opts) = @_;
    if ($opts{bg}) {
        if (my $pid = _bg()) {
            return $pid;
        }
    }
    db_restart();
    exit if $opts{bg};
}

sub status {
    my $self = shift;

    my $info = db_status;
    $info->{ready}   = ($info->{pid} ? 1 : 0);
    # If port is not supplied, then PostgreSQL must not be configured.
    if ($info->{port}) {
        $info->{summary} = "PureMessage Centralized Quarantine, port ".
                           $info->{port} . ", using PostgreSQL";
    }
    else {
        $info->{summary} = "PureMessage Centralized Quarantine is not configured";
    }
    return($info);
}

sub running {
    my $self = shift;
    return db_status()->{running};
}

1;
