#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl

eval 'exec /tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl  -S $0 ${1+"$@"}'
    if 0; # not running under some shell
use strict;
use warnings;
use PureMessage::Version qw($PREFIX);
use PureMessage::Database::PostgreSQL::Config qw(%pg_config);
use ActiveState::Handy qw(run shell_quote);

my $cmd = shift;
print usage() unless $cmd;

my %commands = ('start'      => \&pg_start,
                'stop'       => \&pg_stop,
                'restart'    => \&pg_restart,
                'status'     => \&pg_print_status,
                'dump'       => \&pg_dump,
                'import'     => \&pg_import,
                'fullstatus' => \&pg_print_full_status,
                'vacuum'     => \&pg_vacuum,
                'analyze'    => \&pg_analyze,
                'version'    => \&pg_version,
	       );

my @opts = @ARGV;

my $pg_cmd = "$PREFIX/postgres/bin/pg_ctl";
my @pg_cmd_args = ("-D", "$pg_config{data_dir}");

# Check that we are running as the postgres user
my $pguser = (stat("$PREFIX/postgres"))[4];
if ($> != $pguser) {
    my $id = `id`;
    print "\n\tThis program must be run as the ". getpwuid($pguser)." user.\n";
    print "\tCurrently: $id\n";
    exit(-1);
}

# get and do the action
my $action = $commands{$cmd};
usage() unless $action;
my $fn;
if ($cmd eq "dump" or $cmd eq "import") {
    if (@ARGV == 0) {
        print "The $cmd command requires an additional parameter\n";
        usage();
    }
}

exit &$action(@ARGV);


sub usage {
    (my $prog = $0) =~ s,.*/,,;
    $prog =~ s#\.pl$##;
    print <<USAGE;
Usage: $prog start|stop|restart|status|dump|analyze|import [filename] [opts]

The dump and import commands require a filename parameter.

Any additional parameters are passed to the database control program.
USAGE
    exit;
}

# Determine the PostgreSQL PID
sub pg_pid {
    # Open the status program and parse output for the pid
    # Make sure to read everything so as to avoid a broken pipe.
    open(my $ctl, "$pg_cmd -D $pg_config{data_dir} status |") 
	or die "Failed to run the PostgreSQL control program. $!";
    my $pid = 0;
    while (<$ctl>) {
	chomp;
        if (/is running \(pid: (\d+)\)\s*$/i) {
            $pid = $1;
        }
    }
    close($ctl);
    return $pid; 
}

# is this indeed a PID of a running PostgreSQL server?
sub pg_pid_running {
    my $pid = shift;
    return $pid if ($pid && qx(ps -p$pid) =~ /postmast|postgres/);
    return undef;
}

sub pg_start {
    my $pid = pg_pid();
    my $status = 0;
    if ($pid and pg_pid_running($pid)) {
        print "PostgreSQL is already running.\n";
    }
    else {
        $status = system("$pg_cmd", @pg_cmd_args, "-w", "-o",
                        "-i -p $pg_config{pg_port}", "-l",
                        "$PREFIX/var/log/pg.log", "start", @opts) >> 8;
    }

    return $status;
}

sub pg_stop {
    if (pg_pid()) {
        my $status;
        $status = system("$pg_cmd", 
                         @pg_cmd_args, 
                         qw(stop -w -m fast), 
                         @opts) >> 8;
        if (pg_pid()) {
            $status = system("$pg_cmd", 
                             @pg_cmd_args, 
                             qw(stop -m immediate), 
                             @opts) >> 8;
        }
        return $status;
    }
    else {
        print "PostgreSQL is not running.\n";
        return 2;
    }
} 

sub pg_restart {
    my $c = 1;
    pg_stop if pg_pid();
    pg_start;
}

sub pg_status {
    use PureMessage::_sys;
    my %info = ();
    my $pid = pg_pid();
    $info{'port'}     = $pg_config{pg_port};
    $info{'pid'}      = $pid;
    $info{'running'}  = ($pid and pg_pid_running($pid)) ? $info{pid} : 0;
    $info{'hostname'} = PureMessage::_sys::gethostname();
    return \%info;
}

sub pg_print_status {
    my $info = pg_status();
    unless ($$info{'running'}) {
	print "PostgreSQL database is not running.\n";
	return 2;
    }
    print "PostgreSQL Database is running:\n";
    print "   pid:     $$info{'pid'}\n";
    print "  port:     $$info{'port'}\n" if $$info{port};
    return 0;
}


sub pg_print_full_status {
    my $info = pg_status();
    foreach my $k (keys %$info) {
        print "\t$k = ".$info->{$k}."\n";
    }
}

sub pg_dump {
    my $filename = shift @opts;
    die unless defined $filename;

    # Check if we can write to the file
    eval { run("\@touch $filename") };
    unless (-w $filename) {
        my $pguser = getpwuid((stat("$PREFIX/postgres"))[4]);
        print <<EOM;

Error - $filename is not writable by the $pguser user.

EOM
        return -1;
    }

    run("\@-$PREFIX/postgres/bin/pg_dump -p $pg_config{pg_port} "
        . shell_quote($pg_config{db_name}, @opts)
	. " > $filename");
    return ($? >> 8);
}

sub pg_import {
    my $filename = shift @opts;
    die "Error - can't import $filename: $!" unless defined $filename 
                                                    and -r $filename;

    run("\@-$PREFIX/postgres/bin/psql -p $pg_config{pg_port} "
        . shell_quote($pg_config{db_name}, @opts)
	. " < $filename");
    return ($? >> 8);
}


sub pg_vacuum {
    # free up unused disk space and generate statistics
    run("\@-$PREFIX/postgres/bin/vacuumdb "
        . shell_quote('-v', '-d', $pg_config{db_name}, '-p',
		      $pg_config{pg_port}, '-z', @opts)
	. " >> $PREFIX/postgres/vacuum.log 2>&1");
    return ($? >> 8);
}

sub pg_analyze {
    # generate statistics
    run("\@-$PREFIX/postgres/bin/psql "
        . shell_quote('-d', $pg_config{db_name}, '-p',
		      $pg_config{pg_port}, '-c', "analyze"));
    return ($? >> 8);
}

sub pg_version {
    # Open the status program and parse output for the version
    # Make sure to read everything so as to avoid a broken pipe.
    open(my $ctl, "$pg_cmd --version |") 
	or die "Failed to run the PostgreSQL control program. $!";
    my $ver = '0.0.0';
    while (<$ctl>) {
	chomp;
        if (/(\d+\.\d+\.\d+)$/i) {
            $ver = $1;
        }
    }
    close($ctl);
    print "$ver\n";
    return 0;
}

__END__

=head1 NAME

pmx-database-pg - PostgreSQL control application

=head1 SYNOPSIS

    pmx-database-pg start
    pmx-database-pg stop
    pmx-database-pg restart
    pmx-database-pg status
    pmx-database-pg analyze
    pmx-database-pg dump   <filename>
    pmx-database-pg import <filename>
    pmx-database-pg vacuum

=head1 DESCRIPTION

C<pmx-database-pg> is a control program for the PureMessage installation of
the PostgreSQL Database server.  It is used to perform the regular operations
on the PostgreSQL server.  This program is to be run by the owner of the 
<pmx-prefix>/postgres directory.

The following sub-commands are recognized:

=over

=item B<pmx-database-pg start>

Start PostgreSQL if it is not already running.

=item B<pmx-database-pg stop>

Stop PostgreSQL.

=item B<pmx-database-pg restart>

Restart PostgreSQL.

=item B<pmx-database-pg status>

Print status information about PostgreSQL.

=item B<pmx-database-pg dump>

Dump the database contents to the specified file.

=item B<pmx-database-pg import>

Import data from the specified file into the database.

=item B<pmx-database-pg vacuum>

Perform garbage collection on the PostgreSQL database and generate statistics for query optimizer

=item B<pmx-database-pg analyze>

Generate statistics for query optimizer

=back

=head1 COPYRIGHT

Copyright (c) 2003 ActiveState.  All rights reserved.

=cut
